@@ -2,6 +2,48 @@
 
 #include "cache.h"
 
+//------------------- Waypoint -------------------
+
+Waypoint::Waypoint() {
+  m_type = Unknown;
+}
+
+void Waypoint::setComment(const QString &cmt) {
+  m_comment = cmt;
+}
+
+void Waypoint::setType(const Type &type) {
+  m_type = type;
+}
+
+QString Waypoint::iconFile() const {
+  int i;
+
+  const struct { Waypoint::Type type; QString str; } tags[] = {
+    { Waypoint::Multistage,   "multistage" },
+    { Waypoint::Parking,      "parking" },
+    { Waypoint::Final,        "final" },
+    { Waypoint::Question,     "question" },
+    { Waypoint::Trailhead,    "trailhead" },
+    { Waypoint::Refpoint,     "refpoint" },
+    { Waypoint::Unknown,      "<unknown>" }
+  };
+
+  for(i=0;(tags[i].type!=Waypoint::Unknown) && (tags[i].type != m_type);i++); 
+     
+  return "wpt_" + tags[i].str;
+}
+
+QString Waypoint::comment() const {
+  return m_comment;
+}
+
+QDebug operator<<(QDebug dbg, const Waypoint &w) {
+  dbg.nospace() << "Waypoint " << w.iconFile() << " " << w.name() << " " << w.coordinate() << "\n";
+  dbg.nospace() << "    " << w.description() << " " << w.comment();
+  return dbg;
+}
+
 //------------------- Attribute -------------------
 
 Attribute::Attribute(const Id &id, bool yes) {
@@ -685,6 +727,25 @@ void Cache::clearAttributes() {
   m_attributes.clear();
 }
 
+QList<Waypoint> Cache::waypoints() const {
+  return m_waypoints;
+}
+
+void Cache::appendWaypoint(const Waypoint &waypoint) {
+  m_waypoints.append(waypoint);
+}
+
+void Cache::updateWaypoint(const Waypoint &waypoint) {
+  QList<Waypoint>::iterator i;
+  for( i = m_waypoints.begin(); i != m_waypoints.end(); ++i ) 
+    if(i->name() == waypoint.name())
+      *i = waypoint;
+}
+
+void Cache::clearWaypoints() {
+  m_waypoints.clear();
+}
+
 QDebug operator<<(QDebug debug, const Cache &c) {
   debug.nospace() << "Cache(" << c.name() << 
     ", " << c.coordinate() << ", " << c.description() << ",\n  ";
@@ -706,6 +767,9 @@ QDebug operator<<(QDebug debug, const Cache &c) {
   foreach(Attribute attribute, c.attributes()) 
     debug.nospace() << attribute << endl;
 
+  foreach(Waypoint waypoint, c.waypoints()) 
+    debug.nospace() << waypoint << endl;
+
   return debug;
 }
 
