@@ -29,6 +29,7 @@ class CustomWindow : public QMainWindow
   void setCentralWidget(QWidget *);
   void setWindowTitle(const QString &);
   void addMenuEntry(const QString &, QObject *, const char *);
+  void reParent(CustomWindow *);
 
 #ifdef Q_OS_SYMBIAN
   virtual void MrccatoCommand(TRemConCoreApiOperationId aOperationId,
@@ -57,6 +58,10 @@ class CustomWindow : public QMainWindow
   QVBoxLayout *m_layout;
   QLabel *m_title;
 
+  QList <CustomWindow*> m_children;
+
+  CustomWindow *m_parent;
+
 #ifdef Q_OS_SYMBIAN
   CRemConInterfaceSelector* iInterfaceSelector;
   CRemConCoreApiTarget*     iCoreTarget;
